#pragma check_stack(off)
#pragma comment(linker,"/OPT:NOWIN98")
#pragma comment(lib,"FreeImage.lib")
#include <windows.h>
#define FREEIMAGE_LIB
#include "FreeImage.h"


bool SaveScreenshot(FREE_IMAGE_FORMAT format, char* Filename);
bool ConvertImage(FREE_IMAGE_FORMAT InFormat, FREE_IMAGE_FORMAT OutFormat, char* InFilename, char* OutFilename);

typedef struct {
    DWORD  mVersion;
    HWND   mHwnd;
    BOOL   mKeep;
} LOADINFO;


int WINAPI DllMain(HINSTANCE hInstance,DWORD dwReason,LPVOID Reserved) {
	return 1;
}

int WINAPI UnloadDll(int timeout)
{
	if (!timeout)
	{
		FreeImage_DeInitialise();
	}
	return 0;
}

void WINAPI LoadDll(void*)
{
	FreeImage_Initialise();
}
int __stdcall Screenshot(HWND mWnd, HWND aWnd, char *data, char *parms, BOOL print, BOOL nopause) {
	FREE_IMAGE_FORMAT outFIF = FreeImage_GetFIFFromFilename(data);
	if (!FreeImage_FIFSupportsWriting(outFIF))
		strcpy(data,"E_UNSUPPORTED_OUTPUT_FORMAT");
	else 
		if (SaveScreenshot(outFIF,data))
			strcpy(data,"S_OK");
		else
			strcpy(data,"E_ERROR_CREATING_FILE");
	return 3;
}

int __stdcall Convert(HWND mWnd, HWND aWnd, char *data, char *parms, BOOL print, BOOL nopause) {
	DWORD length = strlen(data);
	bool isOK = false;
	bool isParmsOK = false;
	char*temp = strchr(data,'?');
	if (temp != NULL) {
		char*temp2 = strchr(temp+1,'?');

		if (temp2 != NULL)
			temp2[0] = '\0';

		isParmsOK = true;
		temp[0] = '\0';
		temp++;
		FREE_IMAGE_FORMAT inFIF = FreeImage_GetFIFFromFilename(data);
		FREE_IMAGE_FORMAT outFIF = FreeImage_GetFIFFromFilename(temp);
		BOOL isinFIFgood = FreeImage_FIFSupportsReading(inFIF);
		BOOL isoutFIFgood = FreeImage_FIFSupportsWriting(outFIF);
		
		if (!isinFIFgood) 
			if (!isoutFIFgood) 
				strcpy(data,"E_UNSUPPORTED_INPUT_AND_OUTPUT_FORMAT");
			else 
				strcpy(data,"E_UNSUPPORTED_INPUT_FORMAT");

		if (!isoutFIFgood) 
			strcpy(data,"E_UNSUPPORTED_OUTPUT_FORMAT");

		if ((!isoutFIFgood) || (!isinFIFgood)) 
			return 3;

		isOK = ConvertImage(inFIF,outFIF,data,temp);
	}
	if (!isParmsOK) {
		strcpy(data,"E_INVALID_PARAMETERS");
	}
	else if (!isOK) {
		strcpy(data,"E_ERROR_CONVERTING");
	}
	else {
		strcpy(data,"S_OK");
	}
	return 3;
}

bool SaveScreenshot(FREE_IMAGE_FORMAT format, char* Filename) {
	HBITMAP hBmp;
	HWND hWnd = GetDesktopWindow();
	//get the screen resolution
	RECT myrect;
	GetWindowRect(hWnd,&myrect);
	int width = myrect.right - myrect.left;
	int height = myrect.bottom - myrect.top;
	//get the dc of the desktop/screen
	HDC hScreen = GetDC(hWnd);
	//make a compatible buffer
	HDC hPreview = CreateCompatibleDC(hScreen);
	//create a compatible bitmap as well
	hBmp = CreateCompatibleBitmap(hScreen, width, height);
	//select the bitmap into the buffer
	hBmp = (HBITMAP)SelectObject(hPreview,hBmp);
	//blit the screen to the buffer and into the hbitmap
	BitBlt(hPreview,0,0,width,height,hScreen,0,0,SRCCOPY);
	//select the bitmap back out
	hBmp = (HBITMAP)SelectObject(hPreview,hBmp);
	BITMAP bmp;
	GetObject(hBmp, sizeof(BITMAP), (LPSTR)&bmp);
	int cClrBits = (WORD)(bmp.bmPlanes * bmp.bmBitsPixel);
	FIBITMAP *dib = FreeImage_Allocate(bmp.bmWidth, bmp.bmHeight, cClrBits );
	FIBITMAP *dibJPG;
	int retval = GetDIBits(hPreview, hBmp, 0, bmp.bmHeight, FreeImage_GetBits(dib), FreeImage_GetInfo(dib), DIB_RGB_COLORS);
	if (format == FIF_JPEG) {
		dibJPG = FreeImage_ConvertTo24Bits(dib);
		if (dibJPG == NULL) {
			dibJPG = dib;
		}
		else {
			FreeImage_Free(dib);	
		}
		dib = dibJPG;
		FreeImage_Save(FIF_JPEG, dib, Filename, JPEG_QUALITYGOOD);
	}
	else if (format == FIF_PNG) {
		FreeImage_Save(FIF_PNG, dib, Filename, PNG_DEFAULT);
	}
	else if (format == FIF_BMP) {
		FreeImage_Save(FIF_PNG, dib, Filename, BMP_DEFAULT);
	}
	FreeImage_Free(dib);
	DeleteObject(hBmp);
	DeleteDC(hPreview);
	ReleaseDC(hWnd,hScreen);
	return true;
}

bool ConvertImage(FREE_IMAGE_FORMAT InFormat, FREE_IMAGE_FORMAT OutFormat, char* InFilename, char* OutFilename) {
	if ((InFormat == FIF_UNKNOWN) && (OutFormat == FIF_UNKNOWN)) {
		return false;
	}
	FIBITMAP*inImage = FreeImage_Load(InFormat,InFilename,0);
	FIBITMAP*inJPG;
	if (OutFormat == FIF_JPEG) {
		inJPG = FreeImage_ConvertTo24Bits(inImage);
		if (inJPG != NULL) {
			FreeImage_Free(inImage);
			inImage = inJPG;
		}
	}
	if (!inImage) return false;
	if (!FreeImage_Save(OutFormat,inImage,OutFilename,0)) return false;
	FreeImage_Free(inImage);
	return true;
}
